@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,17 +49,17 @@
 
 package com.openexchange.jsieve.export;
 
-import static com.openexchange.mailfilter.services.MailFilterServletServiceRegistry.getServiceRegistry;
 import java.io.BufferedOutputStream;
 import java.io.BufferedReader;
 import java.io.IOException;
 import java.io.InputStreamReader;
+import java.io.PushbackReader;
+import java.io.Reader;
 import java.io.UnsupportedEncodingException;
 import java.net.InetSocketAddress;
 import java.net.Socket;
 import java.net.UnknownHostException;
 import java.nio.charset.UnsupportedCharsetException;
-import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -72,11 +72,12 @@ import javax.security.sasl.SaslClient;
 import javax.security.sasl.SaslException;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.java.Charsets;
-import com.openexchange.java.StringAllocator;
 import com.openexchange.jsieve.export.exceptions.OXSieveHandlerException;
 import com.openexchange.jsieve.export.exceptions.OXSieveHandlerInvalidCredentialsException;
 import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.mailfilter.internal.MailFilterProperties;
+import com.openexchange.mailfilter.services.Services;
+import com.openexchange.tools.encoding.Hex;
 
 /**
  * This class is used to deal with the communication with sieve. For a description of the communication system to sieve see
@@ -233,7 +234,7 @@ public class SieveHandler {
      */
     public void initializeConnection() throws IOException, OXSieveHandlerException, UnsupportedEncodingException, OXSieveHandlerInvalidCredentialsException {
         measureStart();
-        final ConfigurationService config = getServiceRegistry().getService(ConfigurationService.class);
+        final ConfigurationService config = Services.getService(ConfigurationService.class);
 
         useSIEVEResponseCodes = Boolean.parseBoolean(config.getProperty(MailFilterProperties.Values.USE_SIEVE_RESPONSE_CODES.property));
 
@@ -353,7 +354,7 @@ public class SieveHandler {
         String useAuth = "PLAIN";
         final boolean preferGSSAPI;
         {
-            final ConfigurationService service = getServiceRegistry().getService(ConfigurationService.class);
+            final ConfigurationService service = Services.getService(ConfigurationService.class);
             preferGSSAPI = null != service && service.getBoolProperty("com.openexchange.mail.filter.preferGSSAPI", false);
         }
         if (preferGSSAPI && sasl.contains("GSSAPI")) {
@@ -395,12 +396,12 @@ public class SieveHandler {
         bos_sieve.write(CRLF.getBytes(com.openexchange.java.Charsets.UTF_8));
         bos_sieve.flush();
 
-        final String actualline = bis_sieve.readLine();
+        String actualline = bis_sieve.readLine();
         if (null != actualline && actualline.startsWith(SIEVE_OK)) {
             return;
         } else if (null != actualline && actualline.startsWith("NO ")) {
             final String errorMessage = parseError(actualline).replaceAll(CRLF, "\n");
-            throw new OXSieveHandlerException(errorMessage, sieve_host, sieve_host_port, parseSIEVEResponse(actualline, errorMessage));
+            throw new OXSieveHandlerException(errorMessage, sieve_host, sieve_host_port, parseSIEVEResponse(actualline, errorMessage)).setParseError(true);
         } else {
             throw new OXSieveHandlerException("Unknown response code", sieve_host, sieve_host_port, parseSIEVEResponse(actualline, null));
         }
@@ -471,20 +472,94 @@ public class SieveHandler {
             }
             sb.ensureCapacity(parsed[1]);
         }
+        boolean inQuote = false;
+        boolean okStart = false;
+        boolean inComment = false;
         while (true) {
-            final String temp = bis_sieve.readLine();
-            if (null == temp) {
+            int ch = bis_sieve.read();
+            switch (ch) {
+            case -1:
+                // End of stream
                 throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port, null);
-            }
-            if (temp.startsWith(SIEVE_OK)) {
-                if (sb.length() >= 2) {
-                    // We have to strip off the last trailing CRLF...
-                    return sb.substring(0, sb.length() - 2);
+            case '\\':
+                {
+                    okStart = false;
+                    sb.append((char) ch);
+                    final StringBuilder octetBuilder = new StringBuilder();
+                    int limit = 0;
+                    int index = 0;
+                    do {
+                        ch = bis_sieve.read();
+                        if (ch == -1) {
+                            // End of stream
+                            throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port, null);
+                        } else if (ch >= 48 && ch <= 55) {
+                            octetBuilder.append((char)ch);
+                            limit = 3;
+                            index++;
+                        } else {
+                            sb.append((char) ch);
+                        }
+                    } while (index < limit);
+                    if (octetBuilder.length() > 1) {
+                        sb.setLength(sb.length() - 1);
+                        sb.append((char)Integer.parseInt(octetBuilder.toString()));
+                    }
                 }
-                return sb.toString();
+                break;
+            case '"':
+                {
+                    if (!inComment) {
+                        if (inQuote) {
+                            inQuote = false;
+                        } else {
+                            inQuote = true;
+                        }
+                    }
+                    okStart = false;
+                    sb.append((char) ch);
+                }
+                break;
+            case 'O': // OK\r\n
+                {
+                    if (!inQuote) {
+                        okStart = true;
+                    }
+                    sb.append((char) ch);
+                }
+                break;
+            case 'K': // OK\r\n
+                {
+                    if (!inQuote && okStart && !inComment) {
+                        sb.setLength(sb.length() - 1);
+                        consumeUntilCRLF(); // OK "Getscript completed."\r\n
+                        return returnScript(sb);
+                    }
+                    okStart = false;
+                    sb.append((char) ch);
+                }
+                break;
+            case '#': 
+                {
+                    if (!inQuote) {
+                        inComment = true;
+                    }
+                    sb.append((char) ch);
+                }
+                break;
+            case '\n':
+                {
+                    if (inComment) {
+                        inComment = false;
+                    }
+                    sb.append((char) ch);
+                }
+                break;
+            default:
+                okStart = false;
+                sb.append((char) ch);
+                break;
             }
-            sb.append(temp);
-            sb.append(CRLF);
         }
         /*-
          *
@@ -513,6 +588,61 @@ public class SieveHandler {
          */
     }
 
+    private static String returnScript(final StringBuilder sb) {
+        int length = sb.length();
+        if (length >= 2 && sb.charAt(length - 2) == '\r' && sb.charAt(length - 1) == '\n') {
+            // We have to strip off the last trailing CRLF...
+            return sb.substring(0, length - 2);
+        }
+        return sb.toString();
+    }
+
+    private void consumeUntilCRLF() throws IOException, OXSieveHandlerException {
+        Reader in = bis_sieve;
+        boolean doRead = true;
+        int c1 = -1;
+
+        while (doRead && (c1 = in.read()) >= 0) {
+            if (c1 == '\n') {
+                doRead = false;
+            } else if (c1 == '\r') {
+                // Got CR, is the next char LF?
+                boolean twoCRs = false;
+                if (in.markSupported()) {
+                    in.mark(2);
+                }
+                int c2 = in.read();
+                if (c2 == '\r') {
+                    // Discard extraneous CR
+                    twoCRs = true;
+                    c2 = in.read();
+                }
+                if (c2 != '\n') {
+                    // If the reader supports it (which we hope will always be the case), reset to after the first CR.
+                    // Otherwise, we wrap a PushbackReader around the stream so we can unread the characters we don't need.
+                    if (in.markSupported()) {
+                        in.reset();
+                    } else {
+                        if (!(in instanceof PushbackReader)) {
+                            in = new PushbackReader(in, 2);
+                        }
+                        if (c2 != -1) {
+                            ((PushbackReader) in).unread(c2);
+                        }
+                        if (twoCRs) {
+                            ((PushbackReader) in).unread('\r');
+                        }
+                    }
+                }
+                doRead = false;
+            }
+        }
+        if (c1 < 0) {
+            // End of stream
+            throw new OXSieveHandlerException("Communication to SIEVE server aborted. ", sieve_host, sieve_host_port, null);
+        }
+    }
+
     /**
      * Get the list of sieveScripts
      *
@@ -1153,7 +1283,7 @@ public class SieveHandler {
      */
     private static String toString(final String chars, final int start, final int end) {
         final int size = end - start;
-        final StringAllocator theChars = new StringAllocator(size);
+        final StringBuilder theChars = new StringBuilder(size);
         for (int i = 0, j = start; i < size; i++) {
             theChars.append(chars.charAt(j++));
         }
