@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2012 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2014 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -57,11 +57,14 @@ import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Map.Entry;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
+import java.util.regex.Pattern;
 import javax.mail.internet.idn.IDNA;
 import javax.security.auth.Subject;
 import org.apache.jsieve.SieveException;
@@ -71,6 +74,7 @@ import org.apache.jsieve.parser.generated.TokenMgrError;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+import org.json.JSONValue;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.exception.Category;
 import com.openexchange.exception.OXException;
@@ -98,7 +102,7 @@ import com.openexchange.mailfilter.ajax.exceptions.OXMailfilterExceptionCode;
 import com.openexchange.mailfilter.ajax.json.AbstractObject2JSON2Object;
 import com.openexchange.mailfilter.ajax.json.Rule2JSON2Rule;
 import com.openexchange.mailfilter.internal.MailFilterProperties;
-import com.openexchange.mailfilter.services.MailFilterServletServiceRegistry;
+import com.openexchange.mailfilter.services.Services;
 import com.openexchange.session.Session;
 import com.openexchange.tools.net.URIDefaults;
 import com.openexchange.tools.net.URIParser;
@@ -194,7 +198,7 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
     public MailfilterAction(final Subject krbSubject) {
         super();
         this.krbSubject = krbSubject;
-        final ConfigurationService config = MailFilterServletServiceRegistry.getServiceRegistry().getService(
+        final ConfigurationService config = Services.getService(
                 ConfigurationService.class);
         scriptname = config.getProperty(MailFilterProperties.Values.SCRIPT_NAME.property);
         useSIEVEResponseCodes = Boolean.parseBoolean(config.getProperty(MailFilterProperties.Values.USE_SIEVE_RESPONSE_CODES.property));
@@ -279,10 +283,9 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
                     final RuleListAndNextUid rulesandid = sieveTextFilter.readScriptFromString(script);
                     final ClientRulesAndRequire clientrulesandrequire =
                         sieveTextFilter.splitClientRulesAndRequire(rulesandid.getRulelist(), null, rulesandid.isError());
-                    final String body = request.getBody();
-                    final JSONObject json = new JSONObject(body);
+                    final JSONObject json = getJsonBody(request);
 
-                    final ArrayList<Rule> rules = clientrulesandrequire.getRules();
+                    final List<Rule> rules = clientrulesandrequire.getRules();
                     final RuleAndPosition deletedrule =
                         getRightRuleForUniqueId(rules, getUniqueId(json), credentials.getRightUsername(), credentials.getContextString());
                     rules.remove(deletedrule.getPosition());
@@ -352,7 +355,7 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
                     readScriptFromString.getRulelist(),
                     parameters.getParameter(Parameter.FLAG),
                     readScriptFromString.isError());
-                final ArrayList<Rule> clientrules = clientrulesandrequire.getRules();
+                final List<Rule> clientrules = clientrulesandrequire.getRules();
                 changeOutgoingVacationRule(clientrules);
                 return CONVERTER.write(clientrules.toArray(new Rule[clientrules.size()]));
             } catch (final UnsupportedEncodingException e) {
@@ -417,13 +420,11 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
                 final ClientRulesAndRequire clientrulesandrequire =
                     sieveTextFilter.splitClientRulesAndRequire(rules.getRulelist(), null, rules.isError());
 
-                final String body = request.getBody();
-                final JSONObject json = new JSONObject(body);
-                final Rule newrule = CONVERTER.parse(json);
+                final Rule newrule = CONVERTER.parse(getJsonBody(request));
 
                 if (isVacationRule(newrule)) {
                     // A vacation rule...
-                    final ArrayList<Rule> clientrules = clientrulesandrequire.getRules();
+                    final List<Rule> clientrules = clientrulesandrequire.getRules();
                     for (final Rule rule : clientrules) {
                         if (isVacationRule(rule)) {
                             throw OXMailfilterExceptionCode.DUPLICATE_VACATION_RULE.create();
@@ -435,7 +436,7 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
 
                 // Now find the right position inside the array
                 int position = newrule.getPosition();
-                final ArrayList<Rule> clientrules = clientrulesandrequire.getRules();
+                final List<Rule> clientrules = clientrulesandrequire.getRules();
                 if (position >= clientrules.size()) {
                     throw OXMailfilterExceptionCode.BAD_POSITION.create(Integer.valueOf(position));
                 }
@@ -488,6 +489,43 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
         }
     }
 
+    private JSONObject getJsonBody(final MailfilterRequest request) throws JSONException, OXException {
+        final JSONObject jsonObject = new JSONObject(request.getBody());
+        checkJsonValue(jsonObject, null, jsonObject);
+        return jsonObject;
+    }
+
+    private static final Set<String> MUST_NOT_BE_EMPTY = Collections.unmodifiableSet(new HashSet<String>(Arrays.asList("values")));
+
+    private void checkJsonValue(final JSONValue jValue, final String name, final JSONValue parent) throws OXException {
+        if (null != jValue) {
+            if (jValue.isArray()) {
+                final JSONArray jArray = jValue.toArray();
+                final int length = jArray.length();
+                if (0 == length) {
+                    if (null != name && MUST_NOT_BE_EMPTY.contains(name)) {
+                        // SIEVE does not support empty arrays
+                        throw OXMailfilterExceptionCode.INVALID_SIEVE_RULE.create(parent.toString());
+                    }
+                }
+                for (int i = 0; i < length; i++) {
+                    final Object object = jArray.opt(i);
+                    if (object instanceof JSONValue) {
+                        checkJsonValue((JSONValue) object, null, parent);
+                    }
+                }
+            } else if (jValue.isObject()) {
+                final JSONObject jObject = jValue.toObject();
+                for (final Entry<String, Object> entry : jObject.entrySet()) {
+                    final Object object = entry.getValue();
+                    if (object instanceof JSONValue) {
+                        checkJsonValue((JSONValue) object, entry.getKey(), parent);
+                    }
+                }
+            }
+        }
+    }
+
     @Override
     protected void actionReorder(final MailfilterRequest request) throws OXException {
         synchronized (mutex) {
@@ -507,7 +545,7 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
                     final String body = request.getBody();
                     final JSONArray json = new JSONArray(body);
 
-                    final ArrayList<Rule> clientrules = clientrulesandrequire.getRules();
+                    final List<Rule> clientrules = clientrulesandrequire.getRules();
                     for (int i = 0; i < json.length(); i++) {
                         final int uniqueid = json.getInt(i);
                         final RuleAndPosition rightRule =
@@ -582,11 +620,9 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
                     final ClientRulesAndRequire clientrulesandrequire = sieveTextFilter
                         .splitClientRulesAndRequire(rules.getRulelist(),
                             null, rules.isError());
-                    final String body = request.getBody();
-                    final JSONObject json = new JSONObject(body);
+                    final JSONObject json = getJsonBody(request);
                     final Integer uniqueid = getUniqueId(json);
-                    final ArrayList<Rule> clientrules = clientrulesandrequire
-                        .getRules();
+                    final List<Rule> clientrules = clientrulesandrequire.getRules();
                     if (null == uniqueid) {
                         throw OXMailfilterExceptionCode.ID_MISSING.create();
                     }
@@ -747,7 +783,7 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
 
     private SieveHandler connectRight(final Credentials creds) throws OXException {
         final SieveHandler sieveHandler;
-        final ConfigurationService config = MailFilterServletServiceRegistry.getServiceRegistry().getService(
+        final ConfigurationService config = Services.getService(
                 ConfigurationService.class);
 
         final String logintype = config.getProperty(MailFilterProperties.Values.SIEVE_LOGIN_TYPE.property);
@@ -857,7 +893,7 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
     }
 
     private void changeIncomingVacationRule(final Rule newrule) throws SieveException {
-        final ConfigurationService config = MailFilterServletServiceRegistry.getServiceRegistry().getService(ConfigurationService.class);
+        final ConfigurationService config = Services.getService(ConfigurationService.class);
         final String vacationdomains = config.getProperty(MailFilterProperties.Values.VACATION_DOMAINS.property);
 
         if (null != vacationdomains && 0 != vacationdomains.length()) {
@@ -892,8 +928,8 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
         }
     }
 
-    private void changeOutgoingVacationRule(final ArrayList<Rule> clientrules) throws SieveException {
-        final ConfigurationService config = MailFilterServletServiceRegistry.getServiceRegistry().getService(ConfigurationService.class);
+    private void changeOutgoingVacationRule(final List<Rule> clientrules) throws SieveException {
+        final ConfigurationService config = Services.getService(ConfigurationService.class);
         final String vacationdomains = config.getProperty(MailFilterProperties.Values.VACATION_DOMAINS.property);
 
         if (null != vacationdomains && 0 != vacationdomains.length()) {
@@ -954,7 +990,7 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
         return actionarray;
     }
 
-    private RuleAndPosition getRightRuleForUniqueId(final ArrayList<Rule> clientrules, final Integer uniqueid,
+    private RuleAndPosition getRightRuleForUniqueId(final List<Rule> clientrules, final Integer uniqueid,
             final String userName, final String contextStr) throws OXException {
         for (int i = 0; i < clientrules.size(); i++) {
             final Rule rule = clientrules.get(i);
@@ -967,15 +1003,7 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
 
     private static OXException handleSieveException(final SieveException e) {
         final String msg = e.getMessage();
-        final OXException ret = OXMailfilterExceptionCode.SIEVE_ERROR.create(e, msg);
-        if (null != msg) {
-            if (msg.startsWith(OXMailfilterExceptionCode.ERR_PREFIX_REJECTED_ADDRESS)) {
-                ret.setCategory(Category.CATEGORY_USER_INPUT);
-            } else if (msg.startsWith(OXMailfilterExceptionCode.ERR_PREFIX_INVALID_ADDRESS)) {
-                ret.setCategory(Category.CATEGORY_USER_INPUT);
-            }
-        }
-        return ret;
+        return OXMailfilterExceptionCode.SIEVE_ERROR.create(e, msg);
     }
 
     // private int getIndexOfRightRuleForUniqueId(final ArrayList<Rule>
@@ -1124,14 +1152,32 @@ public class MailfilterAction extends AbstractAction<Rule, MailfilterRequest> {
             if( null != code ) {
                 return new OXException(code.getDetailnumber(), code.getMessage(), e.getSieveHost(), Integer.valueOf(e
                     .getSieveHostPort()), credentials.getRightUsername(), credentials.getContextString()).addCategory(sieveResponse2OXCategory(code)).setPrefix("MAIL_FILTER");
-            } else {
-                return OXMailfilterExceptionCode.SIEVE_COMMUNICATION_ERROR.create(e, e.getSieveHost(), Integer.valueOf(e
-                    .getSieveHostPort()), credentials.getRightUsername(), credentials.getContextString());
             }
-        } else {
+
+            if (e.isParseError()) {
+                return OXMailfilterExceptionCode.INVALID_SIEVE_RULE2.create(e, saneMessage(e.getMessage()));
+            }
+
             return OXMailfilterExceptionCode.SIEVE_COMMUNICATION_ERROR.create(e, e.getSieveHost(), Integer.valueOf(e
                 .getSieveHostPort()), credentials.getRightUsername(), credentials.getContextString());
         }
+
+        if (e.isParseError()) {
+            return OXMailfilterExceptionCode.INVALID_SIEVE_RULE2.create(e, saneMessage(e.getMessage()));
+        }
+
+        return OXMailfilterExceptionCode.SIEVE_COMMUNICATION_ERROR.create(e, e.getSieveHost(), Integer.valueOf(e
+            .getSieveHostPort()), credentials.getRightUsername(), credentials.getContextString());
+    }
+
+    private static final Pattern CONTROL = Pattern.compile("[\\x00-\\x1F\\x7F]+");
+
+    private static String saneMessage(final String message) {
+        if (Strings.isEmpty(message)) {
+            return "";
+        }
+
+        return CONTROL.matcher(message).replaceAll(" ");
     }
 
     private static final class Key {
